/******************/
/* RichRemove.cpp */
/******************/

// Remove the "Rich Signature" generated by MS link.exe

#include <windows.h>
#include <stdio.h>

#define MakePtr(cast, ptr, addValue) (cast)((DWORD_PTR)(ptr) + (DWORD_PTR)(addValue))
#define Round(n, r) (((n+(r-1))/r)*r)
#define SIZEOF_STUB 64

bool RichRemove(char *ptrFileName);

int main(int argc, char* argv[])
{
    if ((argc == 1) || (argc > 2))
    {
        printf("Usage: RichRemove <FileName>\n");
        return EXIT_FAILURE;
    } 

    if (RichRemove(argv[1]))
        return EXIT_SUCCESS;
    else
        return EXIT_FAILURE;
}

bool RichRemove(char *ptrFileName)
{
    PIMAGE_DOS_HEADER ptrImageDosHeader;
    PIMAGE_NT_HEADERS ptrImageNtHeaders;
    PIMAGE_FILE_HEADER ptrImageFileHeader;
    PIMAGE_OPTIONAL_HEADER32 ptrImageOptionalHeader32;
    PIMAGE_SECTION_HEADER ptrImageSectionHeader;

    HANDLE hFile;
    unsigned char *hMemory;
    DWORD nNumberOfBytesToRead, nNumberOfBytesRead, 
          nNumberOfBytesToWrite, nNumberOfBytesWritten,
          dwSizeOfSections, dwFileAlignment, 
          dwSizeOfHeaders, dwSaveSizeOfHeaders,
          ptrSaveExeHeader, dwOffset;

    hFile = CreateFile(ptrFileName, GENERIC_READ, 0,
                       NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

    if (hFile == INVALID_HANDLE_VALUE)
    {
        printf("Error: Unable to open file for reading.\n");
        return false;
    }

    nNumberOfBytesToRead = GetFileSize(hFile, NULL);

    hMemory = new unsigned char[nNumberOfBytesToRead];

    ReadFile(hFile, hMemory, nNumberOfBytesToRead, &nNumberOfBytesRead, FALSE);

    CloseHandle(hFile);

    ptrImageDosHeader = MakePtr(PIMAGE_DOS_HEADER, hMemory, 0);

    if (ptrImageDosHeader->e_magic != IMAGE_DOS_SIGNATURE)
    {
        delete hMemory;
        printf("Error: Invalid DOS Signature.\n");
        return false;
    }

    ptrImageNtHeaders = MakePtr(PIMAGE_NT_HEADERS, hMemory, ptrImageDosHeader->e_lfanew);

    if (ptrImageNtHeaders->Signature != IMAGE_NT_SIGNATURE)
    {
        delete hMemory;
        printf("Error: Invalid NT Signature.\n");
        return false;
    }

    ptrImageFileHeader = (PIMAGE_FILE_HEADER)&ptrImageNtHeaders->FileHeader;
    ptrImageOptionalHeader32 = (PIMAGE_OPTIONAL_HEADER32)&ptrImageNtHeaders->OptionalHeader;

    dwSizeOfSections = ptrImageFileHeader->NumberOfSections * IMAGE_SIZEOF_SECTION_HEADER; 

    // SizeOfHeaders - The combined size of the MS-DOS stub, the PE header, 
    //                 and the section headers, rounded to a multiple of the 
    //                 value specified in the FileAlignment member.

    // 64 + 64 + 20 + 224 + dwSizeOfSections

    dwSizeOfHeaders = sizeof(IMAGE_DOS_HEADER) + SIZEOF_STUB +
                      sizeof(IMAGE_FILE_HEADER) + sizeof(IMAGE_OPTIONAL_HEADER32) + 
                      dwSizeOfSections;
    dwFileAlignment = ptrImageOptionalHeader32->FileAlignment;
    dwSizeOfHeaders = Round(dwSizeOfHeaders, dwFileAlignment);

    dwSaveSizeOfHeaders = ptrImageOptionalHeader32->SizeOfHeaders;
    ptrImageOptionalHeader32->SizeOfHeaders = dwSizeOfHeaders;

    ptrSaveExeHeader = ptrImageDosHeader->e_lfanew;
    ptrImageDosHeader->e_lfanew = 0x00000080;

    // Update PointerToRawData in section tables.

    ptrImageSectionHeader = IMAGE_FIRST_SECTION(ptrImageNtHeaders);

    for (int i = 0; i < ptrImageFileHeader->NumberOfSections; i++)
    {
        ptrImageSectionHeader[i].PointerToRawData = ptrImageSectionHeader[i].PointerToRawData - (dwSaveSizeOfHeaders - dwSizeOfHeaders); 
    }

    // Three writes to put it all back together.

    hFile = CreateFile(ptrFileName, GENERIC_WRITE, 0,
                       NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);

    if (hFile == INVALID_HANDLE_VALUE)
    {
        printf("Error: Unable to open file for writing.\n");
        return false;
    }

    dwOffset = 0;
    nNumberOfBytesToWrite = 128;
    WriteFile(hFile, hMemory + dwOffset, nNumberOfBytesToWrite, &nNumberOfBytesWritten, NULL);

    dwOffset = ptrSaveExeHeader;
    nNumberOfBytesToWrite = dwSizeOfHeaders - 128;
    WriteFile(hFile, hMemory + dwOffset, nNumberOfBytesToWrite, &nNumberOfBytesWritten, NULL);

    dwOffset = dwSaveSizeOfHeaders;
    nNumberOfBytesToWrite = nNumberOfBytesToRead - dwSaveSizeOfHeaders;
    WriteFile(hFile, hMemory + dwOffset, nNumberOfBytesToWrite, &nNumberOfBytesWritten, NULL);

    CloseHandle(hFile);

    delete hMemory;
  
    return true;
}

